
#include "mysqlcppapi/Connection.h"
#include "mysqlcppapi/query/Query.h"
#include "mysqlcppapi/row/Row.h"

namespace mysqlcppapi
{


Connection::Connection()
{
  *m_bLocked = false;
  
  *m_bSuccess = false;

  *m_strHost = "localhost";
  *m_uiPort = 0;
  *m_uiClientFlags = 0;
  *m_uiTimeout = 60;
}

Connection::Connection(const Connection& src)
: m_sharedptr_connection(src.m_sharedptr_connection)
{
  m_bLocked = src.m_bLocked;

  m_bSuccess = src.m_bSuccess;

  m_strHost = src.m_strHost;
  m_strUser = src.m_strUser;
  m_strPassword = src.m_strPassword;
  m_strDatabase = src.m_strDatabase;
  
  m_uiPort = src.m_uiPort;
  m_strUnixSocket = src.m_strUnixSocket;
  m_uiClientFlags = src.m_uiClientFlags;
  m_uiTimeout = src.m_uiTimeout; 
}

Connection& Connection::operator=(const Connection& src)
{
  m_sharedptr_connection = src.m_sharedptr_connection;
  m_bLocked = src.m_bLocked;

  m_bSuccess = src.m_bSuccess;

  m_strHost = src.m_strHost;
  m_strUser = src.m_strUser;
  m_strPassword = src.m_strPassword;
  m_strDatabase = src.m_strDatabase;
 
  m_uiPort = src.m_uiPort;
  m_strUnixSocket = src.m_strUnixSocket;
  m_uiClientFlags = src.m_uiClientFlags;
  m_uiTimeout = src.m_uiTimeout; 
  return *this;
}

Connection::~Connection()
{
}

bool Connection::is_connected() const
{
  //This must be shared between instance, so let's use the bool that's already shared:
  return m_sharedptr_connection.get_do_2nd_stage_dellocation(); //If it's connected then the 2nd stage (mysql_close) is necessary.
}

/*
Connection::type_sharedptr_connection Connection::cobj()
{
  return m_sharedptr_connection;
}
*/

st_mysql_options Connection::get_options() const
{
  return m_sharedptr_connection.obj()->options;
}

std::string Connection::host_info()
{
  return std::string(mysql_get_host_info(m_sharedptr_connection.obj()));
}

int Connection::proto_info()
{
  return mysql_get_proto_info(m_sharedptr_connection.obj());
}

std::string Connection::server_info()
{
  return std::string(mysql_get_server_info(m_sharedptr_connection.obj()));
}

std::string Connection::stat()
{
  return std::string(mysql_stat(m_sharedptr_connection.obj()));
}

int Connection::affected_rows()
{
  return mysql_affected_rows(m_sharedptr_connection.obj());
}

int Connection::insert_id()
{
 return mysql_insert_id(m_sharedptr_connection.obj());
}
    
Query Connection::create_Query()
{
  return Query(*this);
}

void Connection::close()
{
  check_connection_is_open();

  ::mysql_close(m_sharedptr_connection.obj());
  m_sharedptr_connection.set_do_2nd_stage_dellocation(false); //Don't call mysql_close() during deallocation.
  m_sharedptr_connection.clear(); //It's useless to us now.
}

/*
bool Connection::connected() const
{
  return m_bConnected;
}
*/

bool Connection::success() const
{
  return *m_bSuccess;
}

void Connection::lock() throw(ex_Locked)
{
  if(*m_bLocked)
    throw ex_Locked();

  *m_bLocked = true;
}
 
void Connection::unlock()
{
  *m_bLocked = false;
}

/*
void Connection::purge(MYSQL *m)
{
  mysql_close(m_sharedptr_connection.obj());
}
*/
                   
std::string Connection::error()
{
  const char* pchError = mysql_error(m_sharedptr_connection.obj());

  std::string strError;
  if(pchError)
    strError = pchError;
    
  return strError;
} 

int Connection::errnum()
{
  return mysql_errno(m_sharedptr_connection.obj());
}

int Connection::refresh(unsigned int refresh_options)
{
  return mysql_refresh (m_sharedptr_connection.obj(), refresh_options);
}

int Connection::ping()
{
  return ::mysql_ping(m_sharedptr_connection.obj());
}

int Connection::kill(unsigned long pid)
{
  return ::mysql_kill(m_sharedptr_connection.obj(), pid);
}

std::string Connection::get_client_info()
{
  std::string strResult;
  
  const char* pchTemp = mysql_get_client_info();
  if(pchTemp)
    strResult = pchTemp;
    
  return strResult;
}

bool Connection::reload()
{
  check_connection_is_open();
  
  bool suc = !mysql_reload(m_sharedptr_connection.obj());

  if (!suc)
    throw ex_BadQuery(error());
  
  return suc;
}

bool Connection::shutdown ()
{
  check_connection_is_open();
  
  bool suc = !(mysql_shutdown(m_sharedptr_connection.obj()));

  if (!suc)
    throw ex_BadQuery(error());

  return suc;
}  

void Connection::connect()
{
  if(is_connected())
    throw ex_base("The Connection is already open.");

  lock();

  const char* pchUnixSocket = 0;
  if(m_strUnixSocket->size())
    pchUnixSocket = m_strUnixSocket->c_str();

  m_sharedptr_connection.obj()->options.connect_timeout = m_uiTimeout;

  //std::cout << "Connection::connect() connecting: host=" << *m_strHost << " user=" << *m_strUser << " Password=" << *m_strPassword << std::endl;
 
  //std::cout << "Connection::connect() connecting: host=" << *m_strHost << " user=" << *m_strUser << " Password=" << *m_strPassword << " uiport=" << *m_uiPort << " unixsocket=" << pchUnixSocket << " uiclientflags=" << *m_uiClientFlags << " instance=" << m_sharedptr_connection.obj() << std::endl;
  
  //The MYSQL* in m_sharedptr_connection has already been allocated. mysql_real_connect will initialize it:
  const MYSQL* pMySQL = ::mysql_real_connect(m_sharedptr_connection.obj(), m_strHost->c_str(), m_strUser->c_str(), m_strPassword->c_str(), "", *m_uiPort, pchUnixSocket, *m_uiClientFlags);
 
  unlock();
   
  if(!pMySQL)
    throw ex_BadQuery("mysql_real_connect() failed");

  m_sharedptr_connection.set_do_2nd_stage_dellocation(); //Ask it to call mysql_close() during deallocation, because it has been connected.

}

Connection::type_vecStrings Connection::get_DatabaseNames(const std::string& strPattern /* = "" */)
{
  check_connection_is_open();
  
  type_vecStrings vecResult;

  const char* pchPattern = 0;
  if(!strPattern.empty())
    pchPattern = strPattern.c_str();

  
  MYSQL_RES* pMySQL_Res = ::mysql_list_dbs(m_sharedptr_connection.obj(), pchPattern);
  if(pMySQL_Res)
  {
    Result_Store result(pMySQL_Res);
    pMySQL_Res = 0; //Result frees it.

    for(Result_Store::size_type i = 0; i < result.size(); i++)
    {
      Row rowResult = result.fetch_row();
      if(rowResult.size())
      {
        const std::string& strName = rowResult[0];
        vecResult.push_back(strName);
      }
    }
  }

  return vecResult;
}

Connection::type_vecStrings  Connection::get_TableNames(const std::string& strPattern /* = "" */)
{
  check_connection_is_open();
  
  type_vecStrings vecResult;

  const char* pchPattern = 0;
  if(strPattern.size())
    pchPattern = strPattern.c_str();
    
  MYSQL_RES* pMySQL_Res = ::mysql_list_tables(m_sharedptr_connection.obj(), pchPattern);
  if(pMySQL_Res)
  {
    Result_Store result(pMySQL_Res);
    pMySQL_Res = 0; //Result frees it.

    for(Result_Store::size_type i = 0; i < result.size(); i++)
    {
      Row rowResult = result.fetch_row();
      if(rowResult.size())
      {
        const std::string& strName = rowResult[0];
        vecResult.push_back(strName);
      }
    }
  }

  return vecResult;
}

void Connection::select_database(const std::string& strDatabaseName)
{
  check_connection_is_open();
    
  int iError = ::mysql_select_db(m_sharedptr_connection.obj(), strDatabaseName.c_str());

  if(iError)
    throw ex_BadQuery("mysql_select_db() failed");
}

std::string Connection::info ()
{
  check_connection_is_open();
  
  const char* i = mysql_info(m_sharedptr_connection.obj());
  if (!i)
    return std::string();
  else
    return std::string(i);
}

void Connection::query(const std::string& strQuery)
{
  *m_bSuccess = false;
  lock();

  check_connection_is_open();
     
  *m_bSuccess = !(::mysql_query(m_sharedptr_connection.obj(), strQuery.c_str())); 
  unlock();
  
  if (!(*m_bSuccess))
    throw ex_BadQuery(error());
}
  
Result_NoData Connection::execute(const std::string& strQuery)
{
  query(strQuery);
  
  return Result_NoData(*this);
}

Result_Store Connection::store(const std::string& strQuery)
{
  query(strQuery);

  return Result_Store(::mysql_store_result(m_sharedptr_connection.obj()));
}
  
Result_Use Connection::use(const std::string& strQuery)
{
  query(strQuery);

  return Result_Use(::mysql_use_result(m_sharedptr_connection.obj()), *this);
}

int Connection::read_options(enum mysql_option option, const std::string& arg)
{
  check_connection_is_open();
  
  return ::mysql_options(m_sharedptr_connection.obj(), option, arg.c_str());
}

void Connection::set_Host(const std::string& strVal)
{
    *m_strHost = strVal;
}

void Connection::set_User(const std::string& strVal)
{
    *m_strUser = strVal;
}

void Connection::set_Password(const std::string& strVal)
{
    *m_strPassword = strVal;
}

void Connection::set_Port(unsigned int uiVal)
{
    *m_uiPort = uiVal;
}

void Connection::set_UnixSocket(const std::string& strVal)
{
    *m_strUnixSocket = strVal;
}

void Connection::set_ClientFlags(unsigned int uiVal)
{
    *m_uiClientFlags = uiVal;
}

void Connection::set_Timeout(unsigned int uiVal)
{
  *m_uiTimeout = uiVal;
}

bool Connection::create_database(const std::string& strName)
{
  Result_NoData res = execute( "CREATE DATABASE " + strName );
  return !(res.get_succeeded());
}

bool Connection::drop_database(const std::string& strName)
{
  Result_NoData res = execute( "DROP DATABASE " + strName );
  return !(res.get_succeeded());
}

unsigned long Connection::get_last_auto_increment()
{
  check_connection_is_open();
  
  return mysql_insert_id(m_sharedptr_connection.obj());
}

Fields Connection::get_fields(const std::string& strTableName, const std::string& strPattern /*= ""*/)
{
  check_connection_is_open();
  
  const char* pchPattern = 0; //0 means 'all fields'
  if(strPattern.size())
    pchPattern = strPattern.c_str();
    
  MYSQL_RES* pMySqlRes = ::mysql_list_fields(m_sharedptr_connection.obj(), strTableName.c_str(), pchPattern);
  Fields fields =  Result_Store(pMySqlRes, true).get_fields(); //true = full field info.
  return fields;
}

void Connection::check_connection_is_open() throw(ex_base)
{
  if(!is_connected())
    throw ex_base("The Connection is not open.");
}
  

} //namespace
